#ifndef __BPOP_MAIN_H__
#define __BPOP_MAIN_H__

#include "raylib.h"
#include "raymath.h"
#include "rlgl.h"
#include <stdint.h>
#include <math.h>


typedef enum GameScene
{
    GAME_SCENE_NONE,
    GAME_SCENE_MENU,
    GAME_SCENE_DIFFICULTY_SELECT,
    GAME_SCENE_PLAY,
    GAME_SCENE_GAMEOVER,
    GAME_SCENE_HIGHSCORES,
} GameScene;

typedef enum GameDifficulty
{
    GAME_DIFFICULTY_EASY,
    GAME_DIFFICULTY_NORMAL,
    GAME_DIFFICULTY_HARD,
} GameDifficulty;

#define MAX_BUBBLE_TYPES 3

typedef struct Bubble
{
    uint8_t flagIsActive:1;
    uint8_t sameLevelContact:1;
    uint8_t isOutsideZone:1;
    uint8_t bubbleType:3;
    uint8_t bubbleLevel;

    Vector2 position;
    Vector2 velocity;
    float bubbleMergeCooldown;
    float radius;
    float lifeTime;
} Bubble;

#define MAX_BUBBLES 64
#define MAX_STRIKES 3
#define GAMEOVER_COOLDOWN 2.0f

typedef struct PlayfieldScores
{
    uint8_t bubbleCount;
    uint8_t outsideBubbleCount;
    uint32_t score;
} PlayfieldScores;

typedef struct Playfield
{
    Bubble bubbles[MAX_BUBBLES];
    Matrix transform;
    Vector2 fieldSize;
    float waterLevel;
    float maxHeight;
    float minHeight;
    uint8_t nextBubbleLevel;
    uint8_t nextBubbleType;
    // every placed bubble when a bubble is outside the zone counts
    // as a strike, 3 strikes and it's game over. Strikes are 
    // reset when all bubbles are inside the zone.
    uint8_t strikes;
    float gameoverCooldown;
    uint32_t spawnedBubbleCount;
} Playfield;

typedef struct GameTime
{
    float time;
    float deltaTime;
    float fixedTime;
    float fixedDeltaTime;
} GameTime;

#define COLOR_FROM_HEX(hexValue) (Color){((hexValue) >> 16) & 0xFF, ((hexValue) >> 8) & 0xFF, (hexValue) & 0xFF, 0xFF}
#endif